import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.Map;

import org.java_websocket.WebSocket;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.server.WebSocketServer;

public class SignalingServer extends WebSocketServer {
	
	private Map<String,Game> games = new HashMap<String,Game>();
	
	public static void main(String[] args) {
		InetSocketAddress address = new InetSocketAddress("localhost",443);
		SignalingServer server = new SignalingServer(address);
		server.run();
	}

	public SignalingServer(InetSocketAddress address) {
		super(address);
	}
	
	public void onStart() {
		System.out.println("started server");
	}
	
	public void onOpen(WebSocket socket, ClientHandshake handshake) {
		String path = socket.getResourceDescriptor();
		System.out.println("new connection to "+socket.getRemoteSocketAddress()+", path: "+path);
		if (path.startsWith("/skribbl/host")) {
			Game game = new Game(genUnusedID(),socket);
			games.put(game.id,game);
			socket.send("started "+game.id);
			System.out.println("started game "+game.id);
		}else if(path.startsWith("/skribbl/join/")) {
			Game game = games.get(path.split("/",4)[3]);
			if (game==null) {
				socket.send("nope");
				socket.close(4000,"invalid game code");
			}else {
				game.addClient(socket);
			}
		}else {
			System.err.println("invalid connection attempt \""+path+"\"");
			socket.send("nope");
			socket.close(4000,"invalid request path");
		}
	}
	
	public void onClose(WebSocket socket, int code, String reason, boolean remote) {
		System.out.println("closed " + socket.getRemoteSocketAddress() + " with exit code " + code + " additional info: " + reason);
		Game game = gameWithClient(socket);
		if (game!=null) {
			if (game.host==socket) {
				games.remove(game.id);
				System.out.println("closed game "+game.id);
			}else {
				game.removeClient(socket);
			}
		}
	}
	
	public void onMessage(WebSocket socket, String message) {
		System.out.println("received message from "+socket.getRemoteSocketAddress()+": \""+message+"\"");
		if(message.startsWith("message ")) {
			Game game = gameWithClient(socket);
			if (game==null) {
				socket.close(4000,"not currently in any game");
			}else {
				game.handleMessage(socket,message);
			}
		}else {
			System.err.println("can't interpret message: \""+message+"\"");
		}
	}
	
	public void onMessage(WebSocket socket, ByteBuffer message) {
		socket.close(1003);
	}
	
	public void onError(WebSocket conn, Exception ex) {
		if (conn==null) {
			System.err.print("an error occured: ");
		}else {
			System.err.print("an error occurred on connection " + conn.getRemoteSocketAddress()  + ": ");
		}
		ex.printStackTrace();
	}
	
	private Game gameWithClient(WebSocket socket) {
		for (Game game:games.values()) {
			if (game.host==socket||game.hasClient(socket)) {
				return game;
			}
		}
		return null;
	}
	
	private String genUnusedID(){
		String id = Game.genID();
		while(games.containsKey(id)) {
			id = Game.genID();
		}
		return id;
	}
	
}
